@@ -1,12 +1,12 @@
 //! `cost_tracker` keeps tracking transaction cost per chained accounts as well as for entire block
 //! The main functions are:
-//! - would_fit(&tx_cost), immutable function to test if tx with tx_cost would fit into current block
+//! - would_transaction_fit(&tx_cost), immutable function to test if tx with tx_cost would fit into current block
 //! - add_transaction_cost(&tx_cost), mutable function to accumulate tx_cost to tracker.
 //!
 use {
     crate::{block_cost_limits::*, cost_model::TransactionCost},
-    solana_sdk::{clock::Slot, pubkey::Pubkey, saturating_add_assign},
-    std::{cmp::Ordering, collections::HashMap},
+    solana_sdk::{clock::Slot, pubkey::Pubkey, transaction::SanitizedTransaction},
+    std::collections::HashMap,
 };
 
 const WRITABLE_ACCOUNTS_PER_BLOCK: usize = 512;
@@ -21,6 +21,8 @@ pub enum CostTrackerError {
 
     /// would exceed account max limit
     WouldExceedAccountMaxLimit,
+
+    WouldExceedAccountDataMaxLimit,
 }
 
 #[derive(AbiExample, Debug)]
@@ -32,6 +34,7 @@ pub struct CostTracker {
     block_cost: u64,
     vote_cost: u64,
     transaction_count: u64,
+    account_data_size: u64,
 }
 
 impl Default for CostTracker {
@@ -50,25 +53,12 @@ impl Default for CostTracker {
             block_cost: 0,
             vote_cost: 0,
             transaction_count: 0,
+            account_data_size: 0,
         }
     }
 }
 
 impl CostTracker {
-    pub fn new(account_cost_limit: u64, block_cost_limit: u64, vote_cost_limit: u64) -> Self {
-        assert!(account_cost_limit <= block_cost_limit);
-        assert!(vote_cost_limit <= block_cost_limit);
-        Self {
-            account_cost_limit,
-            block_cost_limit,
-            vote_cost_limit,
-            cost_by_writable_accounts: HashMap::with_capacity(WRITABLE_ACCOUNTS_PER_BLOCK),
-            block_cost: 0,
-            vote_cost: 0,
-            transaction_count: 0,
-        }
-    }
-
     // bench tests needs to reset limits
     pub fn set_limits(
         &mut self,
@@ -81,45 +71,51 @@ impl CostTracker {
         self.vote_cost_limit = vote_cost_limit;
     }
 
-    pub fn try_add(&mut self, tx_cost: &TransactionCost) -> Result<u64, CostTrackerError> {
-        self.would_fit(tx_cost)?;
-        self.add_transaction_cost(tx_cost);
-        Ok(self.block_cost)
+    pub fn would_transaction_fit(
+        &self,
+        transaction: &SanitizedTransaction,
+        tx_cost: &TransactionCost,
+    ) -> Result<(), CostTrackerError> {
+        self.would_fit(
+            &tx_cost.writable_accounts,
+            tx_cost.sum(),
+            tx_cost.account_data_size,
+            transaction,
+        )
     }
 
-    pub fn update_execution_cost(
+    pub fn add_transaction_cost(
         &mut self,
-        estimated_tx_cost: &TransactionCost,
-        actual_execution_units: u64,
+        transaction: &SanitizedTransaction,
+        tx_cost: &TransactionCost,
     ) {
-        let estimated_execution_units = estimated_tx_cost.bpf_execution_cost;
-        match actual_execution_units.cmp(&estimated_execution_units) {
-            Ordering::Equal => (),
-            Ordering::Greater => {
-                self.add_transaction_execution_cost(
-                    estimated_tx_cost,
-                    actual_execution_units - estimated_execution_units,
-                );
-            }
-            Ordering::Less => {
-                self.sub_transaction_execution_cost(
-                    estimated_tx_cost,
-                    estimated_execution_units - actual_execution_units,
-                );
-            }
-        }
+        self.add_transaction(
+            &tx_cost.writable_accounts,
+            tx_cost.sum(),
+            tx_cost.account_data_size,
+            transaction,
+        );
     }
 
-    pub fn remove(&mut self, tx_cost: &TransactionCost) {
-        self.remove_transaction_cost(tx_cost);
-    }
-
-    pub fn block_cost(&self) -> u64 {
-        self.block_cost
-    }
-
-    pub fn transaction_count(&self) -> u64 {
-        self.transaction_count
+    pub fn try_add(
+        &mut self,
+        transaction: &SanitizedTransaction,
+        tx_cost: &TransactionCost,
+    ) -> Result<u64, CostTrackerError> {
+        let cost = tx_cost.sum();
+        self.would_fit(
+            &tx_cost.writable_accounts,
+            cost,
+            tx_cost.account_data_size,
+            transaction,
+        )?;
+        self.add_transaction(
+            &tx_cost.writable_accounts,
+            cost,
+            tx_cost.account_data_size,
+            transaction,
+        );
+        Ok(self.block_cost)
     }
 
     pub fn report_stats(&self, bank_slot: Slot) {
@@ -136,9 +132,14 @@ impl CostTracker {
             ("block_cost", self.block_cost as i64, i64),
             ("vote_cost", self.vote_cost as i64, i64),
             ("transaction_count", self.transaction_count as i64, i64),
-            ("number_of_accounts", self.number_of_accounts() as i64, i64),
+            (
+                "number_of_accounts",
+                self.cost_by_writable_accounts.len() as i64,
+                i64
+            ),
             ("costliest_account", costliest_account.to_string(), String),
             ("costliest_account_cost", costliest_account_cost as i64, i64),
+            ("account_data_size", self.account_data_size, i64),
         );
     }
 
@@ -155,18 +156,22 @@ impl CostTracker {
         (costliest_account, costliest_account_cost)
     }
 
-    fn would_fit(&self, tx_cost: &TransactionCost) -> Result<(), CostTrackerError> {
-        let writable_accounts = &tx_cost.writable_accounts;
-        let cost = tx_cost.sum();
-        let vote_cost = if tx_cost.is_simple_vote { cost } else { 0 };
-
+    fn would_fit(
+        &self,
+        keys: &[Pubkey],
+        cost: u64,
+        account_data_len: u64,
+        transaction: &SanitizedTransaction,
+    ) -> Result<(), CostTrackerError> {
         // check against the total package cost
         if self.block_cost.saturating_add(cost) > self.block_cost_limit {
             return Err(CostTrackerError::WouldExceedBlockMaxLimit);
         }
 
         // if vote transaction, check if it exceeds vote_transaction_limit
-        if self.vote_cost.saturating_add(vote_cost) > self.vote_cost_limit {
+        if transaction.is_simple_vote_transaction()
+            && self.vote_cost.saturating_add(cost) > self.vote_cost_limit
+        {
             return Err(CostTrackerError::WouldExceedVoteMaxLimit);
         }
 
@@ -175,8 +180,12 @@ impl CostTracker {
             return Err(CostTrackerError::WouldExceedAccountMaxLimit);
         }
 
+        if self.account_data_size.saturating_add(account_data_len) > MAX_ACCOUNT_DATA_LEN {
+            return Err(CostTrackerError::WouldExceedAccountDataMaxLimit);
+        }
+
         // check each account against account_cost_limit,
-        for account_key in writable_accounts.iter() {
+        for account_key in keys.iter() {
             match self.cost_by_writable_accounts.get(account_key) {
                 Some(chained_cost) => {
                     if chained_cost.saturating_add(cost) > self.account_cost_limit {
@@ -192,54 +201,26 @@ impl CostTracker {
         Ok(())
     }
 
-    fn add_transaction_cost(&mut self, tx_cost: &TransactionCost) {
-        let cost = tx_cost.sum();
-        self.add_transaction_execution_cost(tx_cost, cost);
-        saturating_add_assign!(self.transaction_count, 1);
-    }
-
-    fn remove_transaction_cost(&mut self, tx_cost: &TransactionCost) {
-        let cost = tx_cost.sum();
-        self.sub_transaction_execution_cost(tx_cost, cost);
-        self.transaction_count = self.transaction_count.saturating_sub(1);
-    }
-
-    /// Apply additional actual execution units to cost_tracker
-    fn add_transaction_execution_cost(&mut self, tx_cost: &TransactionCost, adjustment: u64) {
-        for account_key in tx_cost.writable_accounts.iter() {
+    fn add_transaction(
+        &mut self,
+        keys: &[Pubkey],
+        cost: u64,
+        account_data_size: u64,
+        transaction: &SanitizedTransaction,
+    ) {
+        for account_key in keys.iter() {
             let account_cost = self
                 .cost_by_writable_accounts
                 .entry(*account_key)
                 .or_insert(0);
-            *account_cost = account_cost.saturating_add(adjustment);
+            *account_cost = account_cost.saturating_add(cost);
         }
-        self.block_cost = self.block_cost.saturating_add(adjustment);
-        if tx_cost.is_simple_vote {
-            self.vote_cost = self.vote_cost.saturating_add(adjustment);
+        self.block_cost = self.block_cost.saturating_add(cost);
+        if transaction.is_simple_vote_transaction() {
+            self.vote_cost = self.vote_cost.saturating_add(cost);
         }
-    }
-
-    /// Substract extra execution units from cost_tracker
-    fn sub_transaction_execution_cost(&mut self, tx_cost: &TransactionCost, adjustment: u64) {
-        for account_key in tx_cost.writable_accounts.iter() {
-            let account_cost = self
-                .cost_by_writable_accounts
-                .entry(*account_key)
-                .or_insert(0);
-            *account_cost = account_cost.saturating_sub(adjustment);
-        }
-        self.block_cost = self.block_cost.saturating_sub(adjustment);
-        if tx_cost.is_simple_vote {
-            self.vote_cost = self.vote_cost.saturating_sub(adjustment);
-        }
-    }
-
-    /// count number of none-zero CU accounts
-    fn number_of_accounts(&self) -> usize {
-        self.cost_by_writable_accounts
-            .iter()
-            .map(|(_key, units)| if *units > 0 { 1 } else { 0 })
-            .sum()
+        self.account_data_size = self.account_data_size.saturating_add(account_data_size);
+        self.transaction_count = self.transaction_count.saturating_add(1);
     }
 }
 
@@ -255,12 +236,25 @@ mod tests {
             hash::Hash,
             signature::{Keypair, Signer},
             system_transaction,
-            transaction::{SanitizedTransaction, TransactionError, VersionedTransaction},
+            transaction::{DisabledAddressLoader, VersionedTransaction},
         },
         solana_vote_program::vote_transaction,
         std::{cmp, sync::Arc},
     };
 
+    impl CostTracker {
+        fn new(account_cost_limit: u64, block_cost_limit: u64, vote_cost_limit: u64) -> Self {
+            assert!(account_cost_limit <= block_cost_limit);
+            assert!(vote_cost_limit <= block_cost_limit);
+            Self {
+                account_cost_limit,
+                block_cost_limit,
+                vote_cost_limit,
+                ..Self::default()
+            }
+        }
+    }
+
     fn test_setup() -> (Keypair, Hash) {
         solana_logger::setup();
         let GenesisConfigInfo {
@@ -276,22 +270,19 @@ mod tests {
     fn build_simple_transaction(
         mint_keypair: &Keypair,
         start_hash: &Hash,
-    ) -> (SanitizedTransaction, TransactionCost) {
+    ) -> (SanitizedTransaction, Vec<Pubkey>, u64) {
         let keypair = Keypair::new();
         let simple_transaction = SanitizedTransaction::from_transaction_for_tests(
             system_transaction::transfer(mint_keypair, &keypair.pubkey(), 2, *start_hash),
         );
-        let mut tx_cost = TransactionCost::new_with_capacity(1);
-        tx_cost.bpf_execution_cost = 5;
-        tx_cost.writable_accounts.push(mint_keypair.pubkey());
 
-        (simple_transaction, tx_cost)
+        (simple_transaction, vec![mint_keypair.pubkey()], 5)
     }
 
     fn build_simple_vote_transaction(
         mint_keypair: &Keypair,
         start_hash: &Hash,
-    ) -> (SanitizedTransaction, TransactionCost) {
+    ) -> (SanitizedTransaction, Vec<Pubkey>, u64) {
         let keypair = Keypair::new();
         let transaction = vote_transaction::new_vote_transaction(
             vec![42],
@@ -307,15 +298,10 @@ mod tests {
             VersionedTransaction::from(transaction),
             message_hash,
             Some(true),
-            |_| Err(TransactionError::UnsupportedVersion),
+            &DisabledAddressLoader,
         )
         .unwrap();
-        let mut tx_cost = TransactionCost::new_with_capacity(1);
-        tx_cost.bpf_execution_cost = 10;
-        tx_cost.writable_accounts.push(mint_keypair.pubkey());
-        tx_cost.is_simple_vote = true;
-
-        (vote_transaction, tx_cost)
+        (vote_transaction, vec![mint_keypair.pubkey()], 10)
     }
 
     #[test]
@@ -331,13 +317,12 @@ mod tests {
     #[test]
     fn test_cost_tracker_ok_add_one() {
         let (mint_keypair, start_hash) = test_setup();
-        let (_tx, tx_cost) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost = tx_cost.sum();
+        let (tx, keys, cost) = build_simple_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for one simple transaction
         let mut testee = CostTracker::new(cost, cost, cost);
-        assert!(testee.would_fit(&tx_cost).is_ok());
-        testee.add_transaction_cost(&tx_cost);
+        assert!(testee.would_fit(&keys, cost, 0, &tx).is_ok());
+        testee.add_transaction(&keys, cost, 0, &tx);
         assert_eq!(cost, testee.block_cost);
         assert_eq!(0, testee.vote_cost);
         let (_costliest_account, costliest_account_cost) = testee.find_costliest_account();
@@ -347,37 +332,47 @@ mod tests {
     #[test]
     fn test_cost_tracker_ok_add_one_vote() {
         let (mint_keypair, start_hash) = test_setup();
-        let (_tx, tx_cost) = build_simple_vote_transaction(&mint_keypair, &start_hash);
-        let cost = tx_cost.sum();
+        let (tx, keys, cost) = build_simple_vote_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for one simple transaction
         let mut testee = CostTracker::new(cost, cost, cost);
-        assert!(testee.would_fit(&tx_cost).is_ok());
-        testee.add_transaction_cost(&tx_cost);
+        assert!(testee.would_fit(&keys, cost, 0, &tx).is_ok());
+        testee.add_transaction(&keys, cost, 0, &tx);
         assert_eq!(cost, testee.block_cost);
         assert_eq!(cost, testee.vote_cost);
-        let (_ccostliest_account, costliest_account_cost) = testee.find_costliest_account();
+        let (_costliest_account, costliest_account_cost) = testee.find_costliest_account();
         assert_eq!(cost, costliest_account_cost);
     }
 
+    #[test]
+    fn test_cost_tracker_add_data() {
+        let (mint_keypair, start_hash) = test_setup();
+        let (tx, keys, cost) = build_simple_transaction(&mint_keypair, &start_hash);
+
+        // build testee to have capacity for one simple transaction
+        let mut testee = CostTracker::new(cost, cost, cost);
+        assert!(testee.would_fit(&keys, cost, 0, &tx).is_ok());
+        let old = testee.account_data_size;
+        testee.add_transaction(&keys, cost, 1, &tx);
+        assert_eq!(old + 1, testee.account_data_size);
+    }
+
     #[test]
     fn test_cost_tracker_ok_add_two_same_accounts() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with same signed account
-        let (_tx1, tx_cost1) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost1 = tx_cost1.sum();
-        let (_tx2, tx_cost2) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost2 = tx_cost2.sum();
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx2, keys2, cost2) = build_simple_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for two simple transactions, with same accounts
         let mut testee = CostTracker::new(cost1 + cost2, cost1 + cost2, cost1 + cost2);
         {
-            assert!(testee.would_fit(&tx_cost1).is_ok());
-            testee.add_transaction_cost(&tx_cost1);
+            assert!(testee.would_fit(&keys1, cost1, 0, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, 0, &tx1);
         }
         {
-            assert!(testee.would_fit(&tx_cost2).is_ok());
-            testee.add_transaction_cost(&tx_cost2);
+            assert!(testee.would_fit(&keys2, cost2, 0, &tx2).is_ok());
+            testee.add_transaction(&keys2, cost2, 0, &tx2);
         }
         assert_eq!(cost1 + cost2, testee.block_cost);
         assert_eq!(1, testee.cost_by_writable_accounts.len());
@@ -389,21 +384,19 @@ mod tests {
     fn test_cost_tracker_ok_add_two_diff_accounts() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with diff accounts
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
         let second_account = Keypair::new();
-        let (_tx1, tx_cost1) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost1 = tx_cost1.sum();
-        let (_tx2, tx_cost2) = build_simple_transaction(&second_account, &start_hash);
-        let cost2 = tx_cost2.sum();
+        let (tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
 
         // build testee to have capacity for two simple transactions, with same accounts
         let mut testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2, cost1 + cost2);
         {
-            assert!(testee.would_fit(&tx_cost1).is_ok());
-            testee.add_transaction_cost(&tx_cost1);
+            assert!(testee.would_fit(&keys1, cost1, 0, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, 0, &tx1);
         }
         {
-            assert!(testee.would_fit(&tx_cost2).is_ok());
-            testee.add_transaction_cost(&tx_cost2);
+            assert!(testee.would_fit(&keys2, cost2, 0, &tx2).is_ok());
+            testee.add_transaction(&keys2, cost2, 0, &tx2);
         }
         assert_eq!(cost1 + cost2, testee.block_cost);
         assert_eq!(2, testee.cost_by_writable_accounts.len());
@@ -415,21 +408,19 @@ mod tests {
     fn test_cost_tracker_chain_reach_limit() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with same signed account
-        let (_tx1, tx_cost1) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost1 = tx_cost1.sum();
-        let (_tx2, tx_cost2) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost2 = tx_cost2.sum();
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx2, keys2, cost2) = build_simple_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for two simple transactions, but not for same accounts
         let mut testee = CostTracker::new(cmp::min(cost1, cost2), cost1 + cost2, cost1 + cost2);
         // should have room for first transaction
         {
-            assert!(testee.would_fit(&tx_cost1).is_ok());
-            testee.add_transaction_cost(&tx_cost1);
+            assert!(testee.would_fit(&keys1, cost1, 0, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, 0, &tx1);
         }
         // but no more sapce on the same chain (same signer account)
         {
-            assert!(testee.would_fit(&tx_cost2).is_err());
+            assert!(testee.would_fit(&keys2, cost2, 0, &tx2).is_err());
         }
     }
 
@@ -437,23 +428,21 @@ mod tests {
     fn test_cost_tracker_reach_limit() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with diff accounts
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
         let second_account = Keypair::new();
-        let (_tx1, tx_cost1) = build_simple_transaction(&mint_keypair, &start_hash);
-        let cost1 = tx_cost1.sum();
-        let (_tx2, tx_cost2) = build_simple_transaction(&second_account, &start_hash);
-        let cost2 = tx_cost2.sum();
+        let (tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
 
         // build testee to have capacity for each chain, but not enough room for both transactions
         let mut testee =
             CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2 - 1, cost1 + cost2 - 1);
         // should have room for first transaction
         {
-            assert!(testee.would_fit(&tx_cost1).is_ok());
-            testee.add_transaction_cost(&tx_cost1);
+            assert!(testee.would_fit(&keys1, cost1, 0, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, 0, &tx1);
         }
         // but no more room for package as whole
         {
-            assert!(testee.would_fit(&tx_cost2).is_err());
+            assert!(testee.would_fit(&keys2, cost2, 0, &tx2).is_err());
         }
     }
 
@@ -461,33 +450,54 @@ mod tests {
     fn test_cost_tracker_reach_vote_limit() {
         let (mint_keypair, start_hash) = test_setup();
         // build two mocking vote transactions with diff accounts
+        let (tx1, keys1, cost1) = build_simple_vote_transaction(&mint_keypair, &start_hash);
         let second_account = Keypair::new();
-        let (_tx1, tx_cost1) = build_simple_vote_transaction(&mint_keypair, &start_hash);
-        let (_tx2, tx_cost2) = build_simple_vote_transaction(&second_account, &start_hash);
-        let cost1 = tx_cost1.sum();
-        let cost2 = tx_cost2.sum();
+        let (tx2, keys2, cost2) = build_simple_vote_transaction(&second_account, &start_hash);
 
         // build testee to have capacity for each chain, but not enough room for both votes
         let mut testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2, cost1 + cost2 - 1);
         // should have room for first vote
         {
-            assert!(testee.would_fit(&tx_cost1).is_ok());
-            testee.add_transaction_cost(&tx_cost1);
+            assert!(testee.would_fit(&keys1, cost1, 0, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, 0, &tx1);
         }
         // but no more room for package as whole
         {
-            assert!(testee.would_fit(&tx_cost2).is_err());
+            assert!(testee.would_fit(&keys2, cost2, 0, &tx2).is_err());
         }
         // however there is room for none-vote tx3
         {
             let third_account = Keypair::new();
-            let (_tx3, tx_cost3) = build_simple_transaction(&third_account, &start_hash);
-            assert!(testee.would_fit(&tx_cost3).is_ok());
+            let (tx3, keys3, cost3) = build_simple_transaction(&third_account, &start_hash);
+            assert!(testee.would_fit(&keys3, cost3, 0, &tx3).is_ok());
         }
     }
 
+    #[test]
+    fn test_cost_tracker_reach_data_limit() {
+        let (mint_keypair, start_hash) = test_setup();
+        // build two transactions with diff accounts
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let second_account = Keypair::new();
+        let (tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
+
+        // build testee that passes
+        let testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2 - 1, cost1 + cost2 - 1);
+        assert!(testee
+            .would_fit(&keys1, cost1, MAX_ACCOUNT_DATA_LEN, &tx1)
+            .is_ok());
+        // data is too big
+        assert!(testee
+            .would_fit(&keys2, cost2, MAX_ACCOUNT_DATA_LEN + 1, &tx2)
+            .is_err());
+    }
+
     #[test]
     fn test_cost_tracker_try_add_is_atomic() {
+        let (mint_keypair, start_hash) = test_setup();
+        // build two mocking vote transactions with diff accounts
+        let (tx1, _keys1, _cost1) = build_simple_vote_transaction(&mint_keypair, &start_hash);
+
         let acct1 = Pubkey::new_unique();
         let acct2 = Pubkey::new_unique();
         let acct3 = Pubkey::new_unique();
@@ -505,10 +515,10 @@ mod tests {
         {
             let tx_cost = TransactionCost {
                 writable_accounts: vec![acct1, acct2, acct3],
-                bpf_execution_cost: cost,
+                execution_cost: cost,
                 ..TransactionCost::default()
             };
-            assert!(testee.try_add(&tx_cost).is_ok());
+            assert!(testee.try_add(&tx1, &tx_cost).is_ok());
             let (_costliest_account, costliest_account_cost) = testee.find_costliest_account();
             assert_eq!(cost, testee.block_cost);
             assert_eq!(3, testee.cost_by_writable_accounts.len());
@@ -523,10 +533,10 @@ mod tests {
         {
             let tx_cost = TransactionCost {
                 writable_accounts: vec![acct2],
-                bpf_execution_cost: cost,
+                execution_cost: cost,
                 ..TransactionCost::default()
             };
-            assert!(testee.try_add(&tx_cost).is_ok());
+            assert!(testee.try_add(&tx1, &tx_cost).is_ok());
             let (costliest_account, costliest_account_cost) = testee.find_costliest_account();
             assert_eq!(cost * 2, testee.block_cost);
             assert_eq!(3, testee.cost_by_writable_accounts.len());
@@ -543,10 +553,10 @@ mod tests {
         {
             let tx_cost = TransactionCost {
                 writable_accounts: vec![acct1, acct2],
-                bpf_execution_cost: cost,
+                execution_cost: cost,
                 ..TransactionCost::default()
             };
-            assert!(testee.try_add(&tx_cost).is_err());
+            assert!(testee.try_add(&tx1, &tx_cost).is_err());
             let (costliest_account, costliest_account_cost) = testee.find_costliest_account();
             assert_eq!(cost * 2, testee.block_cost);
             assert_eq!(3, testee.cost_by_writable_accounts.len());
@@ -554,169 +564,4 @@ mod tests {
             assert_eq!(acct2, costliest_account);
         }
     }
-
-    #[test]
-    fn test_adjust_transaction_execution_cost() {
-        let acct1 = Pubkey::new_unique();
-        let acct2 = Pubkey::new_unique();
-        let acct3 = Pubkey::new_unique();
-        let cost = 100;
-        let account_max = cost * 2;
-        let block_max = account_max * 3; // for three accts
-
-        let mut testee = CostTracker::new(account_max, block_max, block_max);
-        let tx_cost = TransactionCost {
-            writable_accounts: vec![acct1, acct2, acct3],
-            bpf_execution_cost: cost,
-            ..TransactionCost::default()
-        };
-        let mut expected_block_cost = tx_cost.sum();
-        let expected_tx_count = 1;
-        assert!(testee.try_add(&tx_cost).is_ok());
-        assert_eq!(expected_block_cost, testee.block_cost());
-        assert_eq!(expected_tx_count, testee.transaction_count());
-        testee
-            .cost_by_writable_accounts
-            .iter()
-            .for_each(|(_key, units)| {
-                assert_eq!(expected_block_cost, *units);
-            });
-
-        // adjust up
-        {
-            let adjustment = 50u64;
-            testee.add_transaction_execution_cost(&tx_cost, adjustment);
-            expected_block_cost += 50;
-            assert_eq!(expected_block_cost, testee.block_cost());
-            assert_eq!(expected_tx_count, testee.transaction_count());
-            testee
-                .cost_by_writable_accounts
-                .iter()
-                .for_each(|(_key, units)| {
-                    assert_eq!(expected_block_cost, *units);
-                });
-        }
-
-        // adjust down
-        {
-            let adjustment = 50u64;
-            testee.sub_transaction_execution_cost(&tx_cost, adjustment);
-            expected_block_cost -= 50;
-            assert_eq!(expected_block_cost, testee.block_cost());
-            assert_eq!(expected_tx_count, testee.transaction_count());
-            testee
-                .cost_by_writable_accounts
-                .iter()
-                .for_each(|(_key, units)| {
-                    assert_eq!(expected_block_cost, *units);
-                });
-        }
-
-        // adjust overflow
-        {
-            testee.add_transaction_execution_cost(&tx_cost, u64::MAX);
-            // expect block cost set to limit
-            assert_eq!(u64::MAX, testee.block_cost());
-            assert_eq!(expected_tx_count, testee.transaction_count());
-            testee
-                .cost_by_writable_accounts
-                .iter()
-                .for_each(|(_key, units)| {
-                    assert_eq!(u64::MAX, *units);
-                });
-        }
-
-        // adjust underflow
-        {
-            testee.sub_transaction_execution_cost(&tx_cost, u64::MAX);
-            // expect block cost set to limit
-            assert_eq!(u64::MIN, testee.block_cost());
-            assert_eq!(expected_tx_count, testee.transaction_count());
-            testee
-                .cost_by_writable_accounts
-                .iter()
-                .for_each(|(_key, units)| {
-                    assert_eq!(u64::MIN, *units);
-                });
-            // assert the number of non-empty accounts is zero, but map
-            // still contains 3 account
-            assert_eq!(0, testee.number_of_accounts());
-            assert_eq!(3, testee.cost_by_writable_accounts.len());
-        }
-    }
-
-    #[test]
-    fn test_update_execution_cost() {
-        let acct1 = Pubkey::new_unique();
-        let acct2 = Pubkey::new_unique();
-        let acct3 = Pubkey::new_unique();
-        let cost = 100;
-
-        let tx_cost = TransactionCost {
-            writable_accounts: vec![acct1, acct2, acct3],
-            bpf_execution_cost: cost,
-            ..TransactionCost::default()
-        };
-
-        let mut cost_tracker = CostTracker::default();
-
-        // Assert OK to add tx_cost
-        assert!(cost_tracker.try_add(&tx_cost).is_ok());
-        let (_costliest_account, costliest_account_cost) = cost_tracker.find_costliest_account();
-        assert_eq!(cost, cost_tracker.block_cost);
-        assert_eq!(cost, costliest_account_cost);
-        assert_eq!(1, cost_tracker.transaction_count);
-
-        // assert no-change if actual units is same as estimated units
-        let mut expected_cost = cost;
-        cost_tracker.update_execution_cost(&tx_cost, cost);
-        let (_costliest_account, costliest_account_cost) = cost_tracker.find_costliest_account();
-        assert_eq!(expected_cost, cost_tracker.block_cost);
-        assert_eq!(expected_cost, costliest_account_cost);
-        assert_eq!(1, cost_tracker.transaction_count);
-
-        // assert cost are adjusted down
-        let reduced_units = 3;
-        expected_cost -= reduced_units;
-        cost_tracker.update_execution_cost(&tx_cost, cost - reduced_units);
-        let (_costliest_account, costliest_account_cost) = cost_tracker.find_costliest_account();
-        assert_eq!(expected_cost, cost_tracker.block_cost);
-        assert_eq!(expected_cost, costliest_account_cost);
-        assert_eq!(1, cost_tracker.transaction_count);
-
-        // assert cost are adjusted up
-        let increased_units = 1;
-        expected_cost += increased_units;
-        cost_tracker.update_execution_cost(&tx_cost, cost + increased_units);
-        let (_costliest_account, costliest_account_cost) = cost_tracker.find_costliest_account();
-        assert_eq!(expected_cost, cost_tracker.block_cost);
-        assert_eq!(expected_cost, costliest_account_cost);
-        assert_eq!(1, cost_tracker.transaction_count);
-    }
-
-    #[test]
-    fn test_remove_transaction_cost() {
-        let mut cost_tracker = CostTracker::default();
-
-        let cost = 100u64;
-        let tx_cost = TransactionCost {
-            writable_accounts: vec![Pubkey::new_unique()],
-            bpf_execution_cost: cost,
-            ..TransactionCost::default()
-        };
-
-        cost_tracker.add_transaction_cost(&tx_cost);
-        // assert cost_tracker is reverted to default
-        assert_eq!(1, cost_tracker.transaction_count);
-        assert_eq!(1, cost_tracker.number_of_accounts());
-        assert_eq!(cost, cost_tracker.block_cost);
-        assert_eq!(0, cost_tracker.vote_cost);
-
-        cost_tracker.remove_transaction_cost(&tx_cost);
-        // assert cost_tracker is reverted to default
-        assert_eq!(0, cost_tracker.transaction_count);
-        assert_eq!(0, cost_tracker.number_of_accounts());
-        assert_eq!(0, cost_tracker.block_cost);
-        assert_eq!(0, cost_tracker.vote_cost);
-    }
 }
