@@ -3,7 +3,7 @@ use {
         accounts::Accounts,
         accounts_db::{
             AccountShrinkThreshold, AccountStorageEntry, AccountsDb, AccountsDbConfig, AppendVecId,
-            BankHashInfo, IndexGenerationInfo, SnapshotStorage,
+            AtomicAppendVecId, BankHashInfo, IndexGenerationInfo, SnapshotStorage,
         },
         accounts_index::AccountSecondaryIndexes,
         accounts_update_notifier_interface::AccountsUpdateNotifier,
@@ -23,7 +23,6 @@ use {
     solana_measure::measure::Measure,
     solana_sdk::{
         clock::{Epoch, Slot, UnixTimestamp},
-        deserialize_utils::default_on_eof,
         epoch_schedule::EpochSchedule,
         fee_calculator::{FeeCalculator, FeeRateGovernor},
         genesis_config::GenesisConfig,
@@ -43,7 +42,7 @@ use {
         },
         thread::Builder,
     },
-    storage::SerializableStorage,
+    storage::{SerializableStorage, SerializedAppendVecId},
 };
 
 mod newer;
@@ -68,12 +67,6 @@ struct AccountsDbFields<T>(
     StoredMetaWriteVersion,
     Slot,
     BankHashInfo,
-    /// all slots that were roots within the last epoch
-    #[serde(deserialize_with = "default_on_eof")]
-    Vec<Slot>,
-    /// slots that were roots within the last epoch for which we care about the hash value
-    #[serde(deserialize_with = "default_on_eof")]
-    Vec<(Slot, Hash)>,
 );
 
 /// Helper type to wrap BufReader streams when deserializing and reconstructing from either just a
@@ -93,9 +86,9 @@ struct SnapshotAccountsDbFields<T> {
 
 impl<T> SnapshotAccountsDbFields<T> {
     /// Collapse the SnapshotAccountsDbFields into a single AccountsDbFields.  If there is no
-    /// incremental snapshot, this returns the AccountsDbFields from the full snapshot.
-    /// Otherwise, use the AccountsDbFields from the incremental snapshot, and a combination
-    /// of the storages from both the full and incremental snapshots.
+    /// incremental snapshot, this returns the AccountsDbFields from the full snapshot.  Otherwise
+    /// this uses the version, slot, and bank hash info from the incremental snapshot, then the
+    /// combination of the storages from both the full and incremental snapshots.
     fn collapse_into(self) -> Result<AccountsDbFields<T>, Error> {
         match self.incremental_snapshot_accounts_db_fields {
             None => Ok(self.full_snapshot_accounts_db_fields),
@@ -104,8 +97,6 @@ impl<T> SnapshotAccountsDbFields<T> {
                 incremental_snapshot_version,
                 incremental_snapshot_slot,
                 incremental_snapshot_bank_hash_info,
-                incremental_snapshot_prior_roots,
-                incremental_snapshot_prior_roots_with_hash,
             )) => {
                 let full_snapshot_storages = self.full_snapshot_accounts_db_fields.0;
                 let full_snapshot_slot = self.full_snapshot_accounts_db_fields.2;
@@ -128,8 +119,6 @@ impl<T> SnapshotAccountsDbFields<T> {
                     incremental_snapshot_version,
                     incremental_snapshot_slot,
                     incremental_snapshot_bank_hash_info,
-                    incremental_snapshot_prior_roots,
-                    incremental_snapshot_prior_roots_with_hash,
                 ))
             }
         }
@@ -376,13 +365,12 @@ fn reconstruct_single_storage<E>(
     slot: &Slot,
     append_vec_path: &Path,
     storage_entry: &E,
-    remapped_append_vec_id: Option<AppendVecId>,
+    append_vec_id: AppendVecId,
     new_slot_storage: &mut HashMap<AppendVecId, Arc<AccountStorageEntry>>,
 ) -> Result<(), Error>
 where
     E: SerializableStorage,
 {
-    let append_vec_id = remapped_append_vec_id.unwrap_or_else(|| storage_entry.id());
     let (accounts, num_accounts) =
         AppendVec::new_from_file(append_vec_path, storage_entry.current_len())?;
     let u_storage_entry =
@@ -430,8 +418,6 @@ where
         snapshot_version,
         snapshot_slot,
         snapshot_bank_hash_info,
-        _snapshot_prior_roots,
-        _snapshot_prior_roots_with_hash,
     ) = snapshot_accounts_db_fields.collapse_into()?;
 
     let snapshot_storages = snapshot_storages.into_iter().collect::<Vec<_>>();
@@ -444,7 +430,7 @@ where
 
     // Remap the deserialized AppendVec paths to point to correct local paths
     let num_collisions = AtomicUsize::new(0);
-    let next_append_vec_id = AtomicUsize::new(0);
+    let next_append_vec_id = AtomicAppendVecId::new(0);
     let mut measure_remap = Measure::start("remap");
     let mut storage = (0..snapshot_storages.len())
         .into_par_iter()
@@ -476,7 +462,7 @@ where
                     //    rename the file to this new path.
                     //    **DEVELOPER NOTE:**  Keep this check last so that it can short-circuit if
                     //    possible.
-                    if storage_entry.id() == remapped_append_vec_id
+                    if storage_entry.id() == remapped_append_vec_id as SerializedAppendVecId
                         || std::fs::metadata(&remapped_append_vec_path).is_err()
                     {
                         break (remapped_append_vec_id, remapped_append_vec_path);
@@ -487,7 +473,7 @@ where
                     num_collisions.fetch_add(1, Ordering::Relaxed);
                 };
                 // Only rename the file if the new ID is actually different from the original.
-                if storage_entry.id() != remapped_append_vec_id {
+                if storage_entry.id() != remapped_append_vec_id as SerializedAppendVecId {
                     std::fs::rename(append_vec_path, &remapped_append_vec_path)?;
                 }
 
@@ -495,7 +481,7 @@ where
                     slot,
                     &remapped_append_vec_path,
                     storage_entry,
-                    Some(remapped_append_vec_id),
+                    remapped_append_vec_id,
                     &mut new_slot_storage,
                 )?;
             }
@@ -527,7 +513,7 @@ where
         .write()
         .unwrap()
         .insert(snapshot_slot, snapshot_bank_hash_info);
-    accounts_db.storage.0.extend(
+    accounts_db.storage.map.extend(
         storage
             .into_iter()
             .map(|(slot, slot_storage_entry)| (slot, Arc::new(RwLock::new(slot_storage_entry)))),
@@ -542,11 +528,11 @@ where
     let mut measure_notify = Measure::start("accounts_notify");
 
     let accounts_db = Arc::new(accounts_db);
-    let accoounts_db_clone = accounts_db.clone();
+    let accounts_db_clone = accounts_db.clone();
     let handle = Builder::new()
         .name("notify_account_restore_from_snapshot".to_string())
         .spawn(move || {
-            accoounts_db_clone.notify_account_restore_from_snapshot();
+            accounts_db_clone.notify_account_restore_from_snapshot();
         })
         .unwrap();
 
